#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require

struct SsaoInput
{
    vec2 uv0;
};

struct FlrInput
{
    vec2 uv0;
};

struct FlrOutput
{
    vec4 outColor;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection

struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};


   

layout(set = 0, binding = 1, std140) uniform PerView
{
    vec4 dynamicResolutionParameters;
} _48;

layout(set = 0, binding = 0, std140) uniform PerFrame
{
    vec4 tempParams;
} _259;

layout(set = 0, binding = 100, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _328;

layout(set = 0, binding = 2) uniform SAMPLER_2D ssaoTex;
layout(set = 0, binding = 4) uniform SAMPLER_2D motionVectorsTex;
layout(set = 0, binding = 3) uniform SAMPLER_2D ssaoPrevTex;
layout(set = 0, binding = 5) uniform SAMPLER_2D depthTex;
layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;

layout(location = 0) in vec2 inUV0;
layout(location = 0) out vec4 outColor;

vec4 _281;

vec4 ssaoTemporal(SsaoInput ssaoInput)
{
    float ao = TEXTURE_LOD(ssaoTex, ssaoInput.uv0, 0.0).x;
    vec2 mv = TEXTURE_LOD(motionVectorsTex, ssaoInput.uv0 * _48.dynamicResolutionParameters.xy, 0.0).xy;
    vec2 prevUV = ssaoInput.uv0 + mv;
    bool _66 = all(greaterThanEqual(prevUV, vec2(0.0)));
    bool _74;
    if (_66)
    {
        _74 = all(lessThanEqual(prevUV, vec2(1.0)));
    }
    else
    {
        _74 = _66;
    }
    if (_74)
    {
        float prevAo = TEXTURE_LOD(ssaoPrevTex, prevUV, 0.0).x;
        ivec2 size = TEXTURE_SIZE(ssaoTex, 0);
        uvec2 wh;
        wh.x = uint(size.x);
        wh.y = uint(size.y);
        vec2 res = vec2(wh);
        vec2 du = vec2(1.0 / float(wh.x), 0.0);
        vec2 dv = vec2(0.0, 1.0 / float(wh.y));
        float currentTopLeft = TEXTURE_LOD(ssaoTex, (ssaoInput.uv0 - dv) - du, 0.0).x;
        float currentTopCenter = TEXTURE_LOD(ssaoTex, ssaoInput.uv0 - dv, 0.0).x;
        float currentTopRight = TEXTURE_LOD(ssaoTex, (ssaoInput.uv0 - dv) + du, 0.0).x;
        float currentMiddleLeft = TEXTURE_LOD(ssaoTex, ssaoInput.uv0 - du, 0.0).x;
        float currentMiddleCenter = TEXTURE_LOD(ssaoTex, ssaoInput.uv0, 0.0).x;
        float currentMiddleRight = TEXTURE_LOD(ssaoTex, ssaoInput.uv0 + du, 0.0).x;
        float currentBottomLeft = TEXTURE_LOD(ssaoTex, (ssaoInput.uv0 + dv) - du, 0.0).x;
        float currentBottomCenter = TEXTURE_LOD(ssaoTex, ssaoInput.uv0 + dv, 0.0).x;
        float currentBottomRight = TEXTURE_LOD(ssaoTex, (ssaoInput.uv0 + dv) + du, 0.0).x;
        float currentMin = min(currentTopLeft, min(currentTopCenter, min(currentTopRight, min(currentMiddleLeft, min(currentMiddleCenter, min(currentMiddleRight, min(currentBottomLeft, min(currentBottomCenter, currentBottomRight))))))));
        float currentMax = max(currentTopLeft, max(currentTopCenter, max(currentTopRight, max(currentMiddleLeft, max(currentMiddleCenter, max(currentMiddleRight, max(currentBottomLeft, max(currentBottomCenter, currentBottomRight))))))));
        float center = (currentMin + currentMax) * 0.5;
        currentMin = ((currentMin - center) * 4.0) + center;
        currentMax = ((currentMax - center) * 4.0) + center;
        prevAo = clamp(prevAo, currentMin, currentMax);
        float temporalWeight = mix(1.0, 0.02500000037252902984619140625, _259.tempParams.x);
        float aoOut = ao;
        if (temporalWeight < 1.0)
        {
            aoOut = mix(prevAo, ao, temporalWeight);
        }
        return vec4(aoOut);
    }
    else
    {
        return vec4(ao);
    }
}

void flrMain(FlrInput flrInput, inout FlrOutput flrOutput)
{
    SsaoInput ssaoInput;
    ssaoInput.uv0 = flrInput.uv0;
    SsaoInput param = ssaoInput;
    flrOutput.outColor = ssaoTemporal(param);
}

void main()
{
    FlrInput flrInput;
    flrInput.uv0 = inUV0;
    FlrInput param = flrInput;
    FlrOutput param_1;
    flrMain(param, param_1);
    FlrOutput flrOutput = param_1;
    outColor = flrOutput.outColor;
}


