//Volumetric lighting on water PS 3. Or not.
#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require

struct FlrInput
{
    vec2 uv0;
    vec4 pos;
};

struct FlrOutput
{
    vec4 emissive;
};

struct SH9Compressed
{
    uvec4 data0[2];
    uvec4 data1;
};

// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection

struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};


   

layout(set = 0, binding = 26, std140) uniform PerMaterial
{
    uvec4 paramIntData0;
    vec4 paramData0;
    vec4 paramData1;
    vec4 paramData2;
    vec4 paramData3;
    vec4 paramData4;
    vec4 paramData5;
    vec4 paramData6;
    vec4 paramData7;
    vec4 paramData8;
    vec4 paramData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
    uvec4 paramImgIDs2;
} _285;

layout(set = 0, binding = 28, std140) uniform PerView
{
    mat4 invViewProjMatrix;
    mat4 invProjMatrix;
    vec4 nearFar;
    vec4 nearFarWidthHeight;
    vec4 cameraPositionWS;
    vec4 dirLightDirWS;
    vec4 dirLightColorAndIntens;
    uvec4 dirLightProjectorParams;
    vec4 dirLightProjectorTransform;
    mat4 dirLightProjectorViewProjMatrix;
    SH9Compressed skyModelSH;
    SH9Compressed globalBackgroundSH;
    mat4 pssmViewProjMatrices[5];
    mat4 localLightViewProjMatrices[16];
    vec4 shadowParameters0;
    vec4 playerWorldPosition;
    vec4 lightingWeights0;
    vec4 lightingWeights1;
    vec4 dynamicResolutionParameters;
    vec4 frameParameters0;
    vec4 frameParameters1;
    vec4 projectionOffsets;
    vec4 irradianceGridWorldPosition;
    vec4 irradianceGridHalfExtents;
    uvec4 irradianceGridResolution;
    mat4 viewMatrix;
    vec4 volumetricsParams2;
} _321;

layout(set = 0, binding = 25, std140) uniform PerInstance
{
    uvec4 params0;
    vec4 materialParams0;
    vec4 materialParams1;
    vec4 materialParams2;
    vec4 materialParams3;
} _390;

layout(set = 0, binding = 100, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _513;

layout(set = 0, binding = 22) uniform SAMPLER_2D flowTex;
layout(set = 0, binding = 20) uniform SAMPLER_2D emissiveTex;
layout(set = 0, binding = 21) uniform SAMPLER_2D emissiveMaskTex;
layout(set = 0, binding = 23) uniform sampler3D scatterBufferTex;
layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;

layout(location = 1) in vec2 inUV0;
layout(location = 6) in vec4 inPosition;
layout(location = 0) out vec4 outColor;
layout(location = 0) in vec3 inNormal;
layout(location = 2) in vec4 inTangent;
layout(location = 3) in vec3 inBinormal;

vec4 computeFlowData(inout vec2 flowSample, float time, float flowSpeed, float flowStrength)
{
    float l0 = length(flowSample);
    if (l0 < 0.00999999977648258209228515625)
    {
        flowSample = vec2(0.0);
    }
    vec2 flow = flowSample * flowStrength;
    vec2 scale = fract(vec2((time * flowSpeed) - 0.5, time * flowSpeed));
    return vec4(flow, scale);
}

vec4 computeFlowData(SAMPLER_2D _flowMap, vec2 _flowMapUV, float time, float flowSpeed, float flowStrength)
{
    vec2 flowSample = (SAMPLE_TEXTURE(_flowMap, _flowMapUV).wy * 2.0) - vec2(1.0);
    vec2 param = flowSample;
    float param_1 = time;
    float param_2 = flowSpeed;
    float param_3 = flowStrength;
    vec4 _150 = computeFlowData(param, param_1, param_2, param_3);
    return _150;
}

vec4 computeFlowmapColor(SAMPLER_2D _inputTex, vec2 _inputUV, vec4 _flowData)
{
    vec2 scale = _flowData.zw;
    float flowLayerBlendFactor = abs((0.5 - scale.x) / 0.5);
    vec4 flowUV = _flowData.xyxy * scale.xxyy;
    vec4 val0 = SAMPLE_TEXTURE(_inputTex, _inputUV + flowUV.xy);
    vec4 val1 = SAMPLE_TEXTURE(_inputTex, _inputUV + flowUV.zw);
    return mix(val0, val1, vec4(flowLayerBlendFactor));
}

vec4 computeFlowmapColor(SAMPLER_2D _flowMap, SAMPLER_2D _inputTex, vec2 _flowMapUV, vec2 _inputUV, float time, float flowSpeed, float flowStrength)
{
    vec2 param = _flowMapUV;
    float param_1 = time;
    float param_2 = flowSpeed;
    float param_3 = flowStrength;
    vec4 flowData = computeFlowData(_flowMap, param, param_1, param_2, param_3);
    vec2 param_4 = _inputUV;
    vec4 param_5 = flowData;
    return computeFlowmapColor(_inputTex, param_4, param_5);
}

vec4 decodeRGBA8uint(uint colorU)
{
    return vec4(float((colorU & 4278190080u) >> uint(24)) * 0.003921560011804103851318359375, float((colorU & 16711680u) >> uint(16)) * 0.003921560011804103851318359375, float((colorU & 65280u) >> uint(8)) * 0.003921560011804103851318359375, float(colorU & 255u) * 0.003921560011804103851318359375);
}

float deviceDepthToLinear(float deviceDepth, float near, float far)
{
    return -((far * near) / ((deviceDepth * (far - near)) + near));
}

float nativeDepthToVolumeZ(float depth, float invDepthExp, float depthRange, vec2 nearFar)
{
    float param = depth;
    float param_1 = nearFar.x;
    float param_2 = nearFar.y;
    float depthLinear = -deviceDepthToLinear(param, param_1, param_2);
    return pow(clamp((depthLinear - nearFar.x) / depthRange, 0.0, 1.0) + 9.9999999392252902907785028219223e-09, invDepthExp);
}

vec3 cellCenterSSToUVW(inout vec3 cellCenter, float invDepthExp, float depthRange, vec2 nearFar)
{
    float param = cellCenter.z;
    float param_1 = invDepthExp;
    float param_2 = depthRange;
    vec2 param_3 = nearFar;
    cellCenter.z = nativeDepthToVolumeZ(param, param_1, param_2, param_3);
    return cellCenter;
}

void flrMain(FlrInput flrInput, inout FlrOutput flrOutput)
{
    vec2 uv = (flrInput.uv0 * _285.paramData0.zw) + _285.paramData0.xy;
    if ((_285.paramIntData0.x & 1u) > 0u)
    {
        uv = (flrInput.uv0 * _285.paramData0.zw) + (_285.paramData0.xy * _321.frameParameters0.x);
    }
    vec4 emissive = vec4(0.0);
    if ((_285.paramIntData0.x & 2u) > 0u)
    {
        vec2 param = flrInput.uv0;
        vec2 param_1 = uv;
        float param_2 = _321.frameParameters0.x;
        float param_3 = _285.paramData2.z;
        float param_4 = _285.paramData2.w;
        emissive = computeFlowmapColor(flowTex, emissiveTex, param, param_1, param_2, param_3, param_4);
    }
    else
    {
        emissive = SAMPLE_TEXTURE(emissiveTex, uv);
    }
    vec4 emissiveMask = SAMPLE_TEXTURE(emissiveMaskTex, flrInput.uv0);
    vec3 _377 = ((emissive.xyz * emissiveMask.xyz) * _285.paramData1.xyz) * _285.paramData2.x;
    flrOutput.emissive = vec4(_377.x, _377.y, _377.z, flrOutput.emissive.w);
    flrOutput.emissive.w = emissive.w * _285.paramData2.y;
    uint param_5 = floatBitsToUint(_390.materialParams0.w);
    vec3 perInstanceEmissiveTint = decodeRGBA8uint(param_5).xyz;
    vec3 _404 = flrOutput.emissive.xyz * (perInstanceEmissiveTint * _390.materialParams0.y);
    flrOutput.emissive = vec4(_404.x, _404.y, _404.z, flrOutput.emissive.w);
    flrOutput.emissive.w *= _390.materialParams0.x;
    vec3 screenUV = flrInput.pos.xyz / vec3(flrInput.pos.w);
    vec2 _426 = (screenUV.xy * 0.5) + vec2(0.5);
    screenUV = vec3(_426.x, _426.y, screenUV.z);
    vec3 param_6 = screenUV;
    float param_7 = _321.volumetricsParams2.y;
    float param_8 = _321.volumetricsParams2.z;
    vec2 param_9 = _321.nearFar.xy;
    
    //Fix here.
    param_6.x -= vk3d_params[VIEW].stereo.x *(1-vk3d_params[VIEW].stereo.y*param_6.z*0.0)*0.5;
    
    vec3 _443 = cellCenterSSToUVW(param_6, param_7, param_8, param_9);
    vec3 scatteringUVW = _443;
    vec4 volSample = textureLod(scatterBufferTex, scatteringUVW, 0.0);
    float weight = (1.0 - volSample.w) * _321.volumetricsParams2.w;
    vec3 _466 = mix(flrOutput.emissive.xyz, volSample.xyz, vec3(weight));
    flrOutput.emissive = vec4(_466.x, _466.y, _466.z, flrOutput.emissive.w);
}

void main()
{
    FlrInput flrInput;
    flrInput.uv0 = inUV0;
    flrInput.pos = inPosition;
    FlrInput param = flrInput;
    FlrOutput param_1;
    flrMain(param, param_1);
    FlrOutput flrOutput = param_1;
    outColor = flrOutput.emissive;
}



