#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require

struct FlrInput
{
    vec3 normal;
    vec3 binormal;
    vec4 tangent;
    vec2 uv0;
    vec4 color;
    vec4 pos;
    vec3 posWS;
    vec4 prevPos;
};

struct FlrOutput
{
    vec4 albedo;
    vec4 normals;
    vec4 parameter;
    uint materialId;
    uint miscParams0;
    vec2 motionVector;
    vec3 normalsTS;
};

struct TexturePatchData
{
    uvec4 index;
    vec4 transform;
    vec4 uvScaleOffset;
};

struct MaterialData
{
    uvec4 paramIntData0;
    vec4 paramFloatData0;
    vec4 paramFloatData1;
    vec4 paramFloatData2;
    vec4 paramFloatData3;
    vec4 paramFloatData4;
    vec4 paramFloatData5;
    vec4 paramFloatData6;
    vec4 paramFloatData7;
    vec4 paramFloatData8;
    vec4 paramFloatData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
    uvec4 paramImgIDs2;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 0, binding = 31, std140) uniform PerMaterial
{
    uvec4 paramIntData0;
    vec4 tint;
    vec4 parameters0;
    vec4 uvTransform;
    vec4 paramData3;
    vec4 normalParams;
    vec4 paramData5;
    vec4 paramData6;
    vec4 paramData7;
    vec4 paramData8;
    vec4 paramData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
    uvec4 paramImgIDs2;
} _340;

layout(set = 0, binding = 32, std140) uniform PerFrame
{
    vec4 frameParameters0;
    vec4 frameParameters1;
    vec4 projectionOffsets;
} _370;

layout(set = 0, binding = 33, std140) uniform PerView
{
    mat4 viewProjMatrix;
    mat4 prevViewProjMatrix;
    vec4 cameraPositionWS;
} _396;

layout(set = 0, binding = 30, std140) uniform PerInstance
{
    uvec4 params0;
    vec4 materialParams0;
    vec4 materialParams1;
    vec4 materialParams2;
    vec4 materialParams3;
} _435;

layout(set = 0, binding = 8, std430) readonly buffer TexturePatchBuffer
{
    TexturePatchData _TexturePatchBuffer[];
} _734;

layout(set = 0, binding = 9, std430) readonly buffer materialDataBuffer
{
    MaterialData _materialDataBuffer[];
} _757;

layout(set = 0, binding = 100, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _1223;

layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 0, binding = 20) uniform SAMPLER_2D albedoTex;
layout(set = 0, binding = 26) uniform SAMPLER_2D dissolveTex;
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 0, binding = 22) uniform SAMPLER_2D parameterTex;
layout(set = 0, binding = 23) uniform SAMPLER_2D parameterOptional0Tex;
layout(set = 0, binding = 21) uniform SAMPLER_2D normalTex;
layout(set = 0, binding = 24) uniform SAMPLER_2D detailNormalTex;
layout(set = 0, binding = 27) uniform SAMPLER_2D colorCorrectionMaskTex;
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;
layout(set = 0, binding = 25) uniform SAMPLER_2D flowTex;
layout(set = 0, binding = 28) uniform samplerCube interiorCubeTex;
layout(set = 0, binding = 29) uniform samplerCube interiorMaskTex;
layout(set = 0, binding = 60) uniform SAMPLER_2D opacityMaskTex;

layout(location = 1) in vec2 inUV0;
layout(location = 0) in vec3 inNormal;
layout(location = 2) in vec4 inTangent;
layout(location = 3) in vec3 inBinormal;
layout(location = 5) in vec4 inColor;
layout(location = 4) in vec3 inPosWS;
layout(location = 6) in vec4 inPosition;
layout(location = 7) in vec4 inPrevPosition;
layout(location = 0) out vec4 outAlbedo;
layout(location = 1) out vec4 outNormals;
layout(location = 2) out vec4 outParameter;
layout(location = 3) out vec4 outMotionVector;

vec4 flrSampleBindless(sampler samplerState, vec2 uv, uint textureIdx)
{
    return texture(sampler2D(globalTextures[nonuniformEXT(textureIdx)], samplerState), uv);
}

vec3 decodeNormalMap(vec4 normalSample)
{
    vec2 _171 = (normalSample.wy - vec2(0.5)) * 2.0;
    vec3 normal;
    normal = vec3(_171.x, _171.y, normal.z);
    normal.z = sqrt(1.0 - ((normal.x * normal.x) - (normal.y * normal.y)));
    return normal;
}

vec4 flrSampleLevelBindless(sampler samplerState, vec2 uv, float lod, uint textureIdx)
{
    return textureLod(sampler2D(globalTextures[nonuniformEXT(textureIdx)], samplerState), uv, lod);
}

vec3 blendNormalMaps(vec3 base, vec3 blend)
{
    vec3 blendedNormal = normalize(vec3(base.xy + blend.xy, base.z * blend.z));
    return blendedNormal;
}

vec4 decodeRGBA8uint(uint colorU)
{
    return vec4(float((colorU & 4278190080u) >> uint(24)) * 0.003921560011804103851318359375, float((colorU & 16711680u) >> uint(16)) * 0.003921560011804103851318359375, float((colorU & 65280u) >> uint(8)) * 0.003921560011804103851318359375, float(colorU & 255u) * 0.003921560011804103851318359375);
}

vec3 gammaToLinear(vec3 color)
{
    return pow(clamp(color, vec3(0.0), vec3(1.0)), vec3(2.2000000476837158203125));
}

vec2 signNotZero(vec2 v)
{
    return vec2((v.x >= 0.0) ? 1.0 : (-1.0), (v.y >= 0.0) ? 1.0 : (-1.0));
}

vec2 encodeNormal(vec3 v)
{
    vec2 p = v.xy * (1.0 / ((abs(v.x) + abs(v.y)) + abs(v.z)));
    vec2 _147;
    if (v.z <= 0.0)
    {
        vec2 param = p;
        _147 = (vec2(1.0) - abs(p.yx)) * signNotZero(param);
    }
    else
    {
        _147 = p;
    }
    return _147;
}

float getLuminance(vec3 color)
{
    return dot(color, vec3(0.2125999927520751953125, 0.715200006961822509765625, 0.072200000286102294921875));
}

vec4 decodeARGB8uint(uint colorU)
{
    return vec4(float((colorU & 16711680u) >> uint(16)) * 0.003921560011804103851318359375, float((colorU & 65280u) >> uint(8)) * 0.003921560011804103851318359375, float(colorU & 255u) * 0.003921560011804103851318359375, float((colorU & 4278190080u) >> uint(24)) * 0.003921560011804103851318359375);
}

vec2 getMotionVector(vec4 currPos, vec4 prevPos, vec4 projectionOffsets)
{
    vec2 currPosNDC = (currPos.xy / vec2(currPos.w)) + projectionOffsets.xy;
    vec2 prevPosNDC = (prevPos.xy / vec2(prevPos.w)) + projectionOffsets.zw;
    currPosNDC.y = 1.0 - currPosNDC.y;
    prevPosNDC.y = 1.0 - prevPosNDC.y;
    vec2 motionVector = (prevPosNDC - currPosNDC) * vec2(0.5, -0.5);
    motionVector = clamp(motionVector, vec2(-1.0), vec2(1.0));
    return motionVector;
}

void flrMain(inout FlrInput flrInput, inout FlrOutput flrOutput)
{
    vec2 uv = (flrInput.uv0 * _340.uvTransform.xy) + _340.uvTransform.zw;
    if ((_340.paramIntData0.x & 8u) > 0u)
    {
        uv = (flrInput.uv0 * _340.uvTransform.xy) + (_340.uvTransform.zw * _370.frameParameters0.x);
    }
    vec4 albedoValue = SAMPLE_TEXTURE1(albedoTex, uv, _370.frameParameters1.x);
    if ((_340.paramIntData0.x & 32u) > 0u)
    {
        float distToCameraNorm = pow(clamp(distance(_396.cameraPositionWS.xyz, flrInput.posWS) / 200.0, 0.0, 1.0), 10.0);
        if (distToCameraNorm < 1.0)
        {
            float dissolveValue = SAMPLE_TEXTURE(dissolveTex, uv * _340.paramData6.x).x;
            if (distToCameraNorm < dissolveValue)
            {
                discard;
            }
        }
    }
    uvec2 skinAlbedoNormalIDs = uvec2(floatBitsToUint(_435.materialParams3.x) & 65535u, (floatBitsToUint(_435.materialParams3.x) >> uint(16)) & 65535u);
    uvec2 skinParameterDetailNormalIDs = uvec2(floatBitsToUint(_435.materialParams3.y) & 65535u, (floatBitsToUint(_435.materialParams3.y) >> uint(16)) & 65535u);
    vec2 param = uv;
    uint param_1 = skinAlbedoNormalIDs.x;
    vec4 albedoOldValue = flrSampleBindless(linearRepeatSampler, param, param_1);
    albedoValue = mix(albedoValue, albedoOldValue, vec4(_435.materialParams2.w));
    if (floatBitsToUint(_435.materialParams1.x) == 501163121u)
    {
        float dissolveValue_1 = SAMPLE_TEXTURE(dissolveTex, uv * _340.paramData6.x).x;
        vec3 _502 = albedoValue.xyz + ((vec3(0.0, 0.5, 1.0) * float(abs(dissolveValue_1 - _435.materialParams0.z) < 0.100000001490116119384765625)) * 1000.0);
        albedoValue = vec4(_502.x, _502.y, _502.z, albedoValue.w);
        vec3 _509 = min(albedoValue.xyz, vec3(8.0));
        albedoValue = vec4(_509.x, _509.y, _509.z, albedoValue.w);
    }
    float alphaValue = 1.0;
    float alphaCutoff = 1.0;
    if (_435.materialParams0.z < 1.0)
    {
        float dissolveValue_2 = SAMPLE_TEXTURE(dissolveTex, uv * _340.paramData6.x).x;
        alphaCutoff = _435.materialParams0.z;
        alphaValue = alphaCutoff - dissolveValue_2;
    }
    if ((_340.paramIntData0.x & 1u) > 0u)
    {
        alphaCutoff *= (1.0 - _340.parameters0.w);
        alphaCutoff = 1.0 - clamp(alphaCutoff, 0.0, 1.0);
        alphaValue *= albedoValue.w;
        alphaValue -= alphaCutoff;
    }
    if (alphaValue < 0.0)
    {
        discard;
    }
    float _563;
    if ((_340.paramIntData0.x & 128u) > 0u)
    {
        _563 = 1.0 / max(albedoValue.w, 0.00999999977648258209228515625);
    }
    else
    {
        _563 = 1.0;
    }
    vec3 _575 = albedoValue.xyz * _563;
    albedoValue = vec4(_575.x, _575.y, _575.z, albedoValue.w);
    vec4 parameterValue = SAMPLE_TEXTURE1(parameterTex, uv, _370.frameParameters1.x);
    vec2 param_2 = uv;
    uint param_3 = skinParameterDetailNormalIDs.x;
    vec4 parameterOldValue = flrSampleBindless(linearRepeatSampler, param_2, param_3);
    parameterValue = mix(parameterValue, parameterOldValue, vec4(_435.materialParams2.w));
    vec4 parameterOptional0Value = SAMPLE_TEXTURE1(parameterOptional0Tex, uv, _370.frameParameters1.x);
    float translucency = parameterOptional0Value.x;
    vec4 normalValue = SAMPLE_TEXTURE1(normalTex, uv, _340.normalParams.y + _370.frameParameters1.x);
    vec4 param_4 = normalValue;
    vec3 normal = decodeNormalMap(param_4);
    vec2 param_5 = uv;
    float param_6 = _340.normalParams.y;
    uint param_7 = skinAlbedoNormalIDs.y;
    vec4 normalOldValue = flrSampleLevelBindless(linearRepeatSampler, param_5, param_6, param_7);
    vec4 param_8 = normalOldValue;
    vec3 normalOld = decodeNormalMap(param_8);
    normalOld = mix(vec3(0.0, 0.0, 1.0), normalOld, vec3(_435.materialParams2.w));
    vec3 param_9 = normal;
    vec3 param_10 = normalOld;
    normal = blendNormalMaps(param_9, param_10);
    vec2 _652 = normal.xy * _340.normalParams.x;
    normal = vec3(_652.x, _652.y, normal.z);
    flrOutput.normalsTS = normal;
    if (_340.normalParams.z > 0.0)
    {
        vec2 detailNormalUV = uv * _340.normalParams.w;
        vec4 param_11 = SAMPLE_TEXTURE1(detailNormalTex, detailNormalUV, _370.frameParameters1.x);
        vec3 detailNormal = decodeNormalMap(param_11);
        vec2 _680 = detailNormal.xy * _340.normalParams.z;
        detailNormal = vec3(_680.x, _680.y, detailNormal.z);
        detailNormal = normalize(detailNormal);
        vec3 param_12 = normal;
        vec3 param_13 = detailNormal;
        normal = blendNormalMaps(param_12, param_13);
        vec2 param_14 = detailNormalUV;
        uint param_15 = skinParameterDetailNormalIDs.y;
        vec4 param_16 = flrSampleBindless(linearRepeatSampler, param_14, param_15);
        vec3 detailNormalOld = decodeNormalMap(param_16);
        vec2 _703 = detailNormalOld.xy * _340.normalParams.z;
        detailNormalOld = vec3(_703.x, _703.y, detailNormalOld.z);
        detailNormalOld = normalize(detailNormalOld);
        detailNormalOld = mix(vec3(0.0, 0.0, 1.0), detailNormalOld, vec3(_435.materialParams2.w));
        vec3 param_17 = normal;
        vec3 param_18 = detailNormalOld;
        normal = blendNormalMaps(param_17, param_18);
    }
    TexturePatchData data;
    MaterialData materialData;
    for (uint i = 0u; i < 4u; i++)
    {
        uint _739 = floatBitsToUint(_435.materialParams1.w) + i;
        data.index = _734._TexturePatchBuffer[_739].index;
        data.transform = _734._TexturePatchBuffer[_739].transform;
        data.uvScaleOffset = _734._TexturePatchBuffer[_739].uvScaleOffset;
        uint _761 = clamp(data.index.x, 0u, 4096u);
        materialData.paramIntData0 = _757._materialDataBuffer[_761].paramIntData0;
        materialData.paramFloatData0 = _757._materialDataBuffer[_761].paramFloatData0;
        materialData.paramFloatData1 = _757._materialDataBuffer[_761].paramFloatData1;
        materialData.paramFloatData2 = _757._materialDataBuffer[_761].paramFloatData2;
        materialData.paramFloatData3 = _757._materialDataBuffer[_761].paramFloatData3;
        materialData.paramFloatData4 = _757._materialDataBuffer[_761].paramFloatData4;
        materialData.paramFloatData5 = _757._materialDataBuffer[_761].paramFloatData5;
        materialData.paramFloatData6 = _757._materialDataBuffer[_761].paramFloatData6;
        materialData.paramFloatData7 = _757._materialDataBuffer[_761].paramFloatData7;
        materialData.paramFloatData8 = _757._materialDataBuffer[_761].paramFloatData8;
        materialData.paramFloatData9 = _757._materialDataBuffer[_761].paramFloatData9;
        materialData.paramImgIDs0 = _757._materialDataBuffer[_761].paramImgIDs0;
        materialData.paramImgIDs1 = _757._materialDataBuffer[_761].paramImgIDs1;
        materialData.paramImgIDs2 = _757._materialDataBuffer[_761].paramImgIDs2;
        vec4 transform = data.transform;
        uvec4 textureIndices = materialData.paramImgIDs0;
        vec2 finalFeatureUv = flrInput.uv0;
        finalFeatureUv = (finalFeatureUv * transform.xy) - transform.zw;
        finalFeatureUv += vec2(0.5);
        vec2 featureUvOffset = abs(vec2(0.5) - finalFeatureUv);
        if (all(lessThanEqual(featureUvOffset, vec2(0.5))))
        {
            if (data.index.x != 0u)
            {
                vec2 param_19 = finalFeatureUv;
                uint param_20 = textureIndices.x;
                vec4 patchAlbedo = flrSampleBindless(linearRepeatSampler, param_19, param_20);
                vec2 param_21 = finalFeatureUv;
                uint param_22 = textureIndices.y;
                vec4 patchNormal = flrSampleBindless(linearRepeatSampler, param_21, param_22);
                vec2 param_23 = finalFeatureUv;
                uint param_24 = textureIndices.z;
                vec4 patchParameter = flrSampleBindless(linearRepeatSampler, param_23, param_24);
                finalFeatureUv = (finalFeatureUv * vec2(1.0)) + vec2(0.0);
                vec4 param_25 = patchNormal;
                vec3 patchNormalDecoded = decodeNormalMap(param_25);
                vec2 _865 = patchNormalDecoded.xy * materialData.paramFloatData4.z;
                patchNormalDecoded = vec3(_865.x, _865.y, patchNormalDecoded.z);
                patchNormalDecoded = normalize(patchNormalDecoded);
                uint param_26 = floatBitsToUint(_435.materialParams1.z);
                vec3 param_27 = decodeRGBA8uint(param_26).xyz;
                vec3 patchTint = gammaToLinear(param_27);
                vec3 _882 = patchAlbedo.xyz * patchTint;
                patchAlbedo = vec4(_882.x, _882.y, _882.z, patchAlbedo.w);
                albedoValue = mix(albedoValue, patchAlbedo, vec4(patchAlbedo.w));
                normal = mix(normal, patchNormalDecoded, vec3(patchAlbedo.w));
                parameterValue = mix(parameterValue, patchParameter, vec4(patchAlbedo.w));
            }
        }
    }
    normal = normalize(((flrInput.tangent.xyz * normal.x) + (flrInput.binormal * normal.y)) + (flrInput.normal * normal.z));
    flrOutput.albedo = vec4(albedoValue.xyz.x, albedoValue.xyz.y, albedoValue.xyz.z, flrOutput.albedo.w);
    flrOutput.albedo.w = albedoValue.w;
    vec3 param_28 = normal;
    vec3 param_29 = flrInput.normal;
    flrOutput.normals = vec4(encodeNormal(param_28), encodeNormal(param_29));
    flrOutput.materialId = _435.params0.w;
    flrInput.color = vec4(1.0);
    vec3 colorCorrectionValue = SAMPLE_TEXTURE(colorCorrectionMaskTex, uv).xyz;
    float correctionMask = clamp(colorCorrectionValue.x * flrInput.color.w, 0.0, 1.0);
    uint param_30 = floatBitsToUint(_435.materialParams2.x);
    vec3 param_31 = decodeRGBA8uint(param_30).xyz;
    vec3 tintPerInstance = gammaToLinear(param_31);
    vec3 perInstanceColor = tintPerInstance * flrOutput.albedo.xyz;
    vec3 _987 = mix(flrOutput.albedo.xyz, perInstanceColor, vec3(correctionMask)) * _435.materialParams2.z;
    flrOutput.albedo = vec4(_987.x, _987.y, _987.z, flrOutput.albedo.w);
    vec3 correctedAlbedo = flrInput.color.zyx * flrOutput.albedo.xyz;
    vec3 param_32 = correctedAlbedo;
    float lum = getLuminance(param_32);
    correctedAlbedo = vec3(lum) + ((correctedAlbedo - vec3(lum)) * _340.tint.z);
    uint param_33 = floatBitsToUint(_340.tint.x);
    vec3 param_34 = decodeARGB8uint(param_33).xyz;
    correctedAlbedo = (correctedAlbedo * gammaToLinear(param_34)) * _340.tint.y;
    vec3 _1032 = mix(flrOutput.albedo.xyz, correctedAlbedo, vec3(correctionMask));
    flrOutput.albedo = vec4(_1032.x, _1032.y, _1032.z, flrOutput.albedo.w);
    vec3 parameterScale = _340.parameters0.xyz;
    flrOutput.parameter = vec4(parameterValue.xyz, 1.0);
    flrOutput.parameter.x = clamp((flrOutput.parameter.x * clamp(parameterScale.x, 0.0, 1.0)) + clamp(parameterScale.x - 1.0, 0.0, 1.0), 0.0, 1.0);
    flrOutput.parameter.y = clamp((flrOutput.parameter.y * clamp(parameterScale.y, 0.0, 1.0)) + clamp(parameterScale.y - 1.0, 0.0, 1.0), 0.0, 1.0);
    flrOutput.parameter.z = 1.0 - clamp((1.0 - flrOutput.parameter.z) * parameterScale.z, 0.0, 1.0);
    flrOutput.parameter.w = translucency;
    vec4 param_35 = flrInput.pos;
    vec4 param_36 = flrInput.prevPos;
    vec4 param_37 = _370.projectionOffsets;
    flrOutput.motionVector = getMotionVector(param_35, param_36, param_37);
    flrOutput.miscParams0 = _435.params0.x & 8191u;
    if ((_340.paramIntData0.x & 1024u) > 0u)
    {
        flrOutput.miscParams0 |= 16384u;
    }
    flrOutput.miscParams0 |= 32768u;
}

float encodeMaterialId(uint materialID)
{
    return unpackSnorm2x16(materialID).x;
}

float packUInt16(uint i)
{
    return 1.52587890625e-05 * float(i);
}

void main()
{
    bool _1128;
    if (!gl_FrontFacing)
    {
        _1128 = (_340.paramIntData0.x & 2u) == 0u;
    }
    else
    {
        _1128 = gl_FrontFacing;
    }
    float normalSign = _1128 ? 1.0 : (-1.0);
    FlrInput flrInput;
    flrInput.uv0 = inUV0;
    flrInput.normal = inNormal * normalSign;
    flrInput.tangent = inTangent;
    vec3 _1149 = flrInput.tangent.xyz * normalSign;
    flrInput.tangent = vec4(_1149.x, _1149.y, _1149.z, flrInput.tangent.w);
    flrInput.binormal = inBinormal * normalSign;
    flrInput.color = inColor;
    flrInput.posWS = inPosWS;
    flrInput.pos = inPosition;
    flrInput.prevPos = inPrevPosition;
    FlrInput param = flrInput;
    FlrOutput param_1;
    flrMain(param, param_1);
    FlrOutput flrOutput = param_1;
    outAlbedo = flrOutput.albedo;
    outNormals = flrOutput.normals;
    outParameter = flrOutput.parameter;
    outMotionVector = vec4(flrOutput.motionVector.x, flrOutput.motionVector.y, outMotionVector.z, outMotionVector.w);
    uint param_2 = flrOutput.materialId;
    outMotionVector.z = encodeMaterialId(param_2);
    uint param_3 = flrOutput.miscParams0;
    outMotionVector.w = packUInt16(param_3);
}

