#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require

struct FlrInput
{
    vec3 normal;
    vec3 binormal;
    vec4 tangent;
    vec2 uv0;
    vec4 color;
    vec4 pos;
    vec3 posWS;
    vec4 prevPos;
};

struct FlrOutput
{
    vec4 albedo;
    vec4 normals;
    vec4 parameter;
    uint materialId;
    uint miscParams0;
    vec2 motionVector;
    vec3 normalsTS;
};

struct TexturePatchData
{
    uvec4 index;
    vec4 transform;
    vec4 uvScaleOffset;
};

struct MaterialData
{
    uvec4 paramIntData0;
    vec4 paramFloatData0;
    vec4 paramFloatData1;
    vec4 paramFloatData2;
    vec4 paramFloatData3;
    vec4 paramFloatData4;
    vec4 paramFloatData5;
    vec4 paramFloatData6;
    vec4 paramFloatData7;
    vec4 paramFloatData8;
    vec4 paramFloatData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
    uvec4 paramImgIDs2;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 0, binding = 31, std140) uniform PerMaterial
{
    uvec4 paramIntData0;
    vec4 tint;
    vec4 parameters0;
    vec4 uvTransform;
    vec4 paramData3;
    vec4 normalParams;
    vec4 paramData5;
    vec4 paramData6;
    vec4 paramData7;
    vec4 paramData8;
    vec4 paramData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
    uvec4 paramImgIDs2;
} _395;

layout(set = 0, binding = 32, std140) uniform PerFrame
{
    vec4 frameParameters0;
    vec4 frameParameters1;
    vec4 projectionOffsets;
} _425;

layout(set = 0, binding = 30, std140) uniform PerInstance
{
    uvec4 params0;
    vec4 materialParams0;
    vec4 materialParams1;
    vec4 materialParams2;
    vec4 materialParams3;
} _444;

layout(set = 0, binding = 33, std140) uniform PerView
{
    mat4 viewProjMatrix;
    mat4 prevViewProjMatrix;
    vec4 cameraPositionWS;
} _748;

layout(set = 0, binding = 8, std430) readonly buffer TexturePatchBuffer
{
    TexturePatchData _TexturePatchBuffer[];
} _946;

layout(set = 0, binding = 9, std430) readonly buffer materialDataBuffer
{
    MaterialData _materialDataBuffer[];
} _951;

layout(set = 0, binding = 100, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _955;

layout(set = 0, binding = 20) uniform SAMPLER_2D albedoTex;
layout(set = 0, binding = 26) uniform SAMPLER_2D dissolveTex;
layout(set = 0, binding = 22) uniform SAMPLER_2D parameterTex;
layout(set = 0, binding = 23) uniform SAMPLER_2D parameterOptional0Tex;
layout(set = 0, binding = 21) uniform SAMPLER_2D normalTex;
layout(set = 0, binding = 24) uniform SAMPLER_2D detailNormalTex;
layout(set = 0, binding = 27) uniform SAMPLER_2D colorCorrectionMaskTex;
layout(set = 0, binding = 28) uniform samplerCube interiorCubeTex;
layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;
layout(set = 0, binding = 25) uniform SAMPLER_2D flowTex;
layout(set = 0, binding = 29) uniform samplerCube interiorMaskTex;
layout(set = 0, binding = 60) uniform SAMPLER_2D opacityMaskTex;

layout(location = 1) in vec2 inUV0;
layout(location = 0) in vec3 inNormal;
layout(location = 2) in vec4 inTangent;
layout(location = 3) in vec3 inBinormal;
layout(location = 5) in vec4 inColor;
layout(location = 4) in vec3 inPosWS;
layout(location = 6) in vec4 inPosition;
layout(location = 7) in vec4 inPrevPosition;
layout(location = 0) out vec4 outAlbedo;
layout(location = 1) out vec4 outNormals;
layout(location = 2) out vec4 outParameter;
layout(location = 3) out vec4 outMotionVector;

vec3 decodeNormalMap(vec4 normalSample)
{
    vec2 _140 = (normalSample.wy - vec2(0.5)) * 2.0;
    vec3 normal;
    normal = vec3(_140.x, _140.y, normal.z);
    normal.z = sqrt(1.0 - ((normal.x * normal.x) - (normal.y * normal.y)));
    return normal;
}

vec3 blendNormalMaps(vec3 base, vec3 blend)
{
    vec3 blendedNormal = normalize(vec3(base.xy + blend.xy, base.z * blend.z));
    return blendedNormal;
}

vec2 signNotZero(vec2 v)
{
    return vec2((v.x >= 0.0) ? 1.0 : (-1.0), (v.y >= 0.0) ? 1.0 : (-1.0));
}

vec2 encodeNormal(vec3 v)
{
    vec2 p = v.xy * (1.0 / ((abs(v.x) + abs(v.y)) + abs(v.z)));
    vec2 _116;
    if (v.z <= 0.0)
    {
        vec2 param = p;
        _116 = (vec2(1.0) - abs(p.yx)) * signNotZero(param);
    }
    else
    {
        _116 = p;
    }
    return _116;
}

float getLuminance(vec3 color)
{
    return dot(color, vec3(0.2125999927520751953125, 0.715200006961822509765625, 0.072200000286102294921875));
}

vec4 decodeARGB8uint(uint colorU)
{
    return vec4(float((colorU & 16711680u) >> uint(16)) * 0.003921560011804103851318359375, float((colorU & 65280u) >> uint(8)) * 0.003921560011804103851318359375, float(colorU & 255u) * 0.003921560011804103851318359375, float((colorU & 4278190080u) >> uint(24)) * 0.003921560011804103851318359375);
}

vec3 gammaToLinear(vec3 color)
{
    return pow(clamp(color, vec3(0.0), vec3(1.0)), vec3(2.2000000476837158203125));
}

vec3 computeInteriorMappingRayIntersection(vec2 uv, vec3 posWS, vec3 camPosWS, vec3 tangent, vec3 binormal, vec3 normal, vec2 interiorTiling, vec2 interiorOffset, uint cubeFaceID)
{
    vec2 f = fract((uv + interiorOffset) * interiorTiling);
    vec3 rayPos = vec3((f * vec2(2.0)) - vec2(1.0), 1.0);
    rayPos = ((tangent * rayPos.x) + (binormal * rayPos.y)) + (normal * rayPos.z);
    vec3 rayPosection = posWS - camPosWS;
    vec3 id = vec3(1.0) / rayPosection;
    vec3 k = abs(id) - (rayPos * id);
    float kMin = min(min(k.x, k.y), k.z);
    rayPos += (rayPosection * kMin);
    vec3 _293;
    if (cubeFaceID == 1u)
    {
        _293 = vec3(-rayPos.x, rayPos.y, -rayPos.z);
    }
    else
    {
        vec3 _308;
        if (cubeFaceID == 2u)
        {
            _308 = vec3(rayPos.z, rayPos.y, -rayPos.x);
        }
        else
        {
            vec3 _323;
            if (cubeFaceID == 3u)
            {
                _323 = vec3(-rayPos.z, rayPos.y, rayPos.x);
            }
            else
            {
                _323 = rayPos;
            }
            _308 = _323;
        }
        _293 = _308;
    }
    rayPos = _293;
    return rayPos;
}

vec2 getMotionVector(vec4 currPos, vec4 prevPos, vec4 projectionOffsets)
{
    vec2 currPosNDC = (currPos.xy / vec2(currPos.w)) + projectionOffsets.xy;
    vec2 prevPosNDC = (prevPos.xy / vec2(prevPos.w)) + projectionOffsets.zw;
    currPosNDC.y = 1.0 - currPosNDC.y;
    prevPosNDC.y = 1.0 - prevPosNDC.y;
    vec2 motionVector = (prevPosNDC - currPosNDC) * vec2(0.5, -0.5);
    motionVector = clamp(motionVector, vec2(-1.0), vec2(1.0));
    return motionVector;
}

void flrMain(FlrInput flrInput, inout FlrOutput flrOutput)
{
    vec2 uv = (flrInput.uv0 * _395.uvTransform.xy) + _395.uvTransform.zw;
    if ((_395.paramIntData0.x & 8u) > 0u)
    {
        uv = (flrInput.uv0 * _395.uvTransform.xy) + (_395.uvTransform.zw * _425.frameParameters0.x);
    }
    vec4 albedoValue = SAMPLE_TEXTURE1(albedoTex, uv, _425.frameParameters1.x);
    if (floatBitsToUint(_444.materialParams1.x) == 501163121u)
    {
        float dissolveValue = SAMPLE_TEXTURE(dissolveTex, uv * _395.paramData6.x).x;
        vec3 _477 = albedoValue.xyz + ((vec3(0.0, 0.5, 1.0) * float(abs(dissolveValue - _444.materialParams0.z) < 0.100000001490116119384765625)) * 1000.0);
        albedoValue = vec4(_477.x, _477.y, _477.z, albedoValue.w);
        vec3 _484 = min(albedoValue.xyz, vec3(8.0));
        albedoValue = vec4(_484.x, _484.y, _484.z, albedoValue.w);
    }
    float _492;
    if ((_395.paramIntData0.x & 128u) > 0u)
    {
        _492 = 1.0 / max(albedoValue.w, 0.00999999977648258209228515625);
    }
    else
    {
        _492 = 1.0;
    }
    vec3 _504 = albedoValue.xyz * _492;
    albedoValue = vec4(_504.x, _504.y, _504.z, albedoValue.w);
    vec4 parameterValue = SAMPLE_TEXTURE1(parameterTex, uv, _425.frameParameters1.x);
    vec4 parameterOptional0Value = SAMPLE_TEXTURE1(parameterOptional0Tex, uv, _425.frameParameters1.x);
    float translucency = parameterOptional0Value.x;
    vec4 normalValue = SAMPLE_TEXTURE1(normalTex, uv, _395.normalParams.y + _425.frameParameters1.x);
    vec4 param = normalValue;
    vec3 normal = decodeNormalMap(param);
    vec2 _543 = normal.xy * _395.normalParams.x;
    normal = vec3(_543.x, _543.y, normal.z);
    flrOutput.normalsTS = normal;
    if (_395.normalParams.z > 0.0)
    {
        vec2 detailNormalUV = uv * _395.normalParams.w;
        vec4 param_1 = SAMPLE_TEXTURE1(detailNormalTex, detailNormalUV, _425.frameParameters1.x);
        vec3 detailNormal = decodeNormalMap(param_1);
        vec2 _572 = detailNormal.xy * _395.normalParams.z;
        detailNormal = vec3(_572.x, _572.y, detailNormal.z);
        detailNormal = normalize(detailNormal);
        vec3 param_2 = normal;
        vec3 param_3 = detailNormal;
        normal = blendNormalMaps(param_2, param_3);
    }
    normal = normalize(((flrInput.tangent.xyz * normal.x) + (flrInput.binormal * normal.y)) + (flrInput.normal * normal.z));
    flrOutput.albedo = vec4(albedoValue.xyz.x, albedoValue.xyz.y, albedoValue.xyz.z, flrOutput.albedo.w);
    flrOutput.albedo.w = albedoValue.w;
    vec3 param_4 = normal;
    vec3 param_5 = flrInput.normal;
    flrOutput.normals = vec4(encodeNormal(param_4), encodeNormal(param_5));
    flrOutput.materialId = _444.params0.w;
    vec3 colorCorrectionValue = SAMPLE_TEXTURE(colorCorrectionMaskTex, uv).xyz;
    float correctionMask = clamp(colorCorrectionValue.x * flrInput.color.w, 0.0, 1.0);
    vec3 correctedAlbedo = flrInput.color.zyx * flrOutput.albedo.xyz;
    vec3 param_6 = correctedAlbedo;
    float lum = getLuminance(param_6);
    correctedAlbedo = vec3(lum) + ((correctedAlbedo - vec3(lum)) * _395.tint.z);
    uint param_7 = floatBitsToUint(_395.tint.x);
    vec3 param_8 = decodeARGB8uint(param_7).xyz;
    correctedAlbedo = (correctedAlbedo * gammaToLinear(param_8)) * _395.tint.y;
    vec3 _680 = mix(flrOutput.albedo.xyz, correctedAlbedo, vec3(correctionMask));
    flrOutput.albedo = vec4(_680.x, _680.y, _680.z, flrOutput.albedo.w);
    vec3 parameterScale = _395.parameters0.xyz;
    flrOutput.parameter = vec4(parameterValue.xyz, 1.0);
    flrOutput.parameter.x = clamp((flrOutput.parameter.x * clamp(parameterScale.x, 0.0, 1.0)) + clamp(parameterScale.x - 1.0, 0.0, 1.0), 0.0, 1.0);
    flrOutput.parameter.y = clamp((flrOutput.parameter.y * clamp(parameterScale.y, 0.0, 1.0)) + clamp(parameterScale.y - 1.0, 0.0, 1.0), 0.0, 1.0);
    flrOutput.parameter.z = 1.0 - clamp((1.0 - flrOutput.parameter.z) * parameterScale.z, 0.0, 1.0);
    flrOutput.parameter.w = translucency;
    vec2 interiorTiling = _395.paramData9.xy;
    vec2 interiorOffset = _395.paramData9.zw;
    uint cubeFaceID = _395.paramIntData0.w;
    vec2 param_9 = flrInput.uv0;
    vec3 param_10 = flrInput.posWS;
    vec3 param_11 = _748.cameraPositionWS.xyz;
    vec3 param_12 = flrInput.tangent.xyz;
    vec3 param_13 = flrInput.binormal;
    vec3 param_14 = flrInput.normal;
    vec2 param_15 = interiorTiling;
    vec2 param_16 = interiorOffset;
    uint param_17 = cubeFaceID;
    vec3 rayPosition = computeInteriorMappingRayIntersection(param_9, param_10, param_11, param_12, param_13, param_14, param_15, param_16, param_17);
    float refractionMipIdx = flrOutput.parameter.x * 3.0;
    vec3 interiorColor = textureLod(interiorCubeTex, rayPosition, refractionMipIdx).xyz;
    vec3 _799 = mix(interiorColor, flrOutput.albedo.xyz, vec3(flrOutput.albedo.w * 1.0));
    flrOutput.albedo = vec4(_799.x, _799.y, _799.z, flrOutput.albedo.w);
    vec4 param_18 = flrInput.pos;
    vec4 param_19 = flrInput.prevPos;
    vec4 param_20 = _425.projectionOffsets;
    flrOutput.motionVector = getMotionVector(param_18, param_19, param_20);
    flrOutput.miscParams0 = _444.params0.x & 8191u;
    if ((_395.paramIntData0.x & 1024u) > 0u)
    {
        flrOutput.miscParams0 |= 16384u;
    }
    flrOutput.albedo.w *= _444.materialParams0.z;
}

float encodeMaterialId(uint materialID)
{
    return unpackSnorm2x16(materialID).x;
}

float packUInt16(uint i)
{
    return 1.52587890625e-05 * float(i);
}

void main()
{
    bool _848;
    if (!gl_FrontFacing)
    {
        _848 = (_395.paramIntData0.x & 2u) == 0u;
    }
    else
    {
        _848 = gl_FrontFacing;
    }
    float normalSign = _848 ? 1.0 : (-1.0);
    FlrInput flrInput;
    flrInput.uv0 = inUV0;
    flrInput.normal = inNormal * normalSign;
    flrInput.tangent = inTangent;
    vec3 _869 = flrInput.tangent.xyz * normalSign;
    flrInput.tangent = vec4(_869.x, _869.y, _869.z, flrInput.tangent.w);
    flrInput.binormal = inBinormal * normalSign;
    flrInput.color = inColor;
    flrInput.posWS = inPosWS;
    flrInput.pos = inPosition;
    flrInput.prevPos = inPrevPosition;
    FlrInput param = flrInput;
    FlrOutput param_1;
    flrMain(param, param_1);
    FlrOutput flrOutput = param_1;
    outAlbedo = flrOutput.albedo;
    outNormals = flrOutput.normals;
    outParameter = flrOutput.parameter;
    outMotionVector = vec4(flrOutput.motionVector.x, flrOutput.motionVector.y, outMotionVector.z, outMotionVector.w);
    uint param_2 = flrOutput.materialId;
    outMotionVector.z = encodeMaterialId(param_2);
    uint param_3 = flrOutput.miscParams0;
    outMotionVector.w = packUInt16(param_3);
}

