#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require

struct FlrInput
{
    vec3 normal;
    vec3 binormal;
    vec4 tangent;
    vec2 uv0;
    vec4 color;
    vec4 pos;
    vec3 posWS;
    vec4 prevPos;
    float dissolve;
};

struct FlrOutput
{
    vec4 albedo;
    vec4 normals;
    vec4 parameter;
    uint materialId;
    uint miscParams0;
    vec2 motionVector;
    vec3 normalsTS;
};

struct TexturePatchData
{
    uvec4 index;
    vec4 transform;
    vec4 uvScaleOffset;
};

struct MaterialData
{
    uvec4 paramIntData0;
    vec4 paramFloatData0;
    vec4 paramFloatData1;
    vec4 paramFloatData2;
    vec4 paramFloatData3;
    vec4 paramFloatData4;
    vec4 paramFloatData5;
    vec4 paramFloatData6;
    vec4 paramFloatData7;
    vec4 paramFloatData8;
    vec4 paramFloatData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
    uvec4 paramImgIDs2;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 0, binding = 31, std140) uniform PerMaterial
{
    uvec4 paramIntData0;
    vec4 tint;
    vec4 parameters0;
    vec4 uvTransform;
    vec4 paramData3;
    vec4 normalParams;
    vec4 paramData5;
    vec4 paramData6;
    vec4 paramData7;
    vec4 paramData8;
    vec4 paramData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
    uvec4 paramImgIDs2;
} _300;

layout(set = 0, binding = 32, std140) uniform PerFrame
{
    vec4 frameParameters0;
    vec4 frameParameters1;
    vec4 projectionOffsets;
} _330;

layout(set = 0, binding = 33, std140) uniform PerView
{
    mat4 viewProjMatrix;
    mat4 prevViewProjMatrix;
    vec4 cameraPositionWS;
} _358;

layout(set = 0, binding = 30, std140) uniform PerInstance
{
    uvec4 params0;
    vec4 materialParams0;
    vec4 materialParams1;
    vec4 materialParams2;
    vec4 materialParams3;
} _394;

layout(set = 0, binding = 8, std430) readonly buffer TexturePatchBuffer
{
    TexturePatchData _TexturePatchBuffer[];
} _906;

layout(set = 0, binding = 9, std430) readonly buffer materialDataBuffer
{
    MaterialData _materialDataBuffer[];
} _911;

layout(set = 0, binding = 100, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _915;

layout(set = 0, binding = 20) uniform SAMPLER_2D albedoTex;
layout(set = 0, binding = 26) uniform SAMPLER_2D dissolveTex;
layout(set = 0, binding = 22) uniform SAMPLER_2D parameterTex;
layout(set = 0, binding = 23) uniform SAMPLER_2D parameterOptional0Tex;
layout(set = 0, binding = 21) uniform SAMPLER_2D normalTex;
layout(set = 0, binding = 24) uniform SAMPLER_2D detailNormalTex;
layout(set = 0, binding = 27) uniform SAMPLER_2D colorCorrectionMaskTex;
layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;
layout(set = 0, binding = 25) uniform SAMPLER_2D flowTex;
layout(set = 0, binding = 28) uniform samplerCube interiorCubeTex;
layout(set = 0, binding = 29) uniform samplerCube interiorMaskTex;
layout(set = 0, binding = 60) uniform SAMPLER_2D opacityMaskTex;

layout(location = 1) in vec2 inUV0;
layout(location = 0) in vec3 inNormal;
layout(location = 2) in vec4 inTangent;
layout(location = 3) in vec3 inBinormal;
layout(location = 5) in vec4 inColor;
layout(location = 4) in vec3 inPosWS;
layout(location = 6) in vec4 inPosition;
layout(location = 7) in vec4 inPrevPosition;
layout(location = 10) in vec4 inDissolve;
layout(location = 0) out vec4 outAlbedo;
layout(location = 1) out vec4 outNormals;
layout(location = 2) out vec4 outParameter;
layout(location = 3) out vec4 outMotionVector;

vec3 decodeNormalMap(vec4 normalSample)
{
    vec2 _131 = (normalSample.wy - vec2(0.5)) * 2.0;
    vec3 normal;
    normal = vec3(_131.x, _131.y, normal.z);
    normal.z = sqrt(1.0 - ((normal.x * normal.x) - (normal.y * normal.y)));
    return normal;
}

vec3 blendNormalMaps(vec3 base, vec3 blend)
{
    vec3 blendedNormal = normalize(vec3(base.xy + blend.xy, base.z * blend.z));
    return blendedNormal;
}

vec2 signNotZero(vec2 v)
{
    return vec2((v.x >= 0.0) ? 1.0 : (-1.0), (v.y >= 0.0) ? 1.0 : (-1.0));
}

vec2 encodeNormal(vec3 v)
{
    vec2 p = v.xy * (1.0 / ((abs(v.x) + abs(v.y)) + abs(v.z)));
    vec2 _107;
    if (v.z <= 0.0)
    {
        vec2 param = p;
        _107 = (vec2(1.0) - abs(p.yx)) * signNotZero(param);
    }
    else
    {
        _107 = p;
    }
    return _107;
}

vec4 decodeRGBA8uint(uint colorU)
{
    return vec4(float((colorU & 4278190080u) >> uint(24)) * 0.003921560011804103851318359375, float((colorU & 16711680u) >> uint(16)) * 0.003921560011804103851318359375, float((colorU & 65280u) >> uint(8)) * 0.003921560011804103851318359375, float(colorU & 255u) * 0.003921560011804103851318359375);
}

vec3 gammaToLinear(vec3 color)
{
    return pow(clamp(color, vec3(0.0), vec3(1.0)), vec3(2.2000000476837158203125));
}

float getLuminance(vec3 color)
{
    return dot(color, vec3(0.2125999927520751953125, 0.715200006961822509765625, 0.072200000286102294921875));
}

vec4 decodeARGB8uint(uint colorU)
{
    return vec4(float((colorU & 16711680u) >> uint(16)) * 0.003921560011804103851318359375, float((colorU & 65280u) >> uint(8)) * 0.003921560011804103851318359375, float(colorU & 255u) * 0.003921560011804103851318359375, float((colorU & 4278190080u) >> uint(24)) * 0.003921560011804103851318359375);
}

vec2 getMotionVector(vec4 currPos, vec4 prevPos, vec4 projectionOffsets)
{
    vec2 currPosNDC = (currPos.xy / vec2(currPos.w)) + projectionOffsets.xy;
    vec2 prevPosNDC = (prevPos.xy / vec2(prevPos.w)) + projectionOffsets.zw;
    currPosNDC.y = 1.0 - currPosNDC.y;
    prevPosNDC.y = 1.0 - prevPosNDC.y;
    vec2 motionVector = (prevPosNDC - currPosNDC) * vec2(0.5, -0.5);
    motionVector = clamp(motionVector, vec2(-1.0), vec2(1.0));
    return motionVector;
}

void flrMain(FlrInput flrInput, inout FlrOutput flrOutput)
{
    vec2 uv = (flrInput.uv0 * _300.uvTransform.xy) + _300.uvTransform.zw;
    if ((_300.paramIntData0.x & 8u) > 0u)
    {
        uv = (flrInput.uv0 * _300.uvTransform.xy) + (_300.uvTransform.zw * _330.frameParameters0.x);
    }
    vec4 albedoValue = SAMPLE_TEXTURE1(albedoTex, uv, _330.frameParameters1.x);
    if ((_300.paramIntData0.x & 32u) > 0u)
    {
        float distToCameraNorm = pow(clamp(distance(_358.cameraPositionWS.xyz, flrInput.posWS) / 200.0, 0.0, 1.0), 10.0);
        if (distToCameraNorm < 1.0)
        {
            float dissolveValue = SAMPLE_TEXTURE(dissolveTex, uv * _300.paramData6.x).x;
            if (distToCameraNorm < dissolveValue)
            {
                discard;
            }
        }
    }
    if (floatBitsToUint(_394.materialParams1.x) == 501163121u)
    {
        float dissolveValue_1 = SAMPLE_TEXTURE(dissolveTex, uv * _300.paramData6.x).x;
        vec3 _425 = albedoValue.xyz + ((vec3(0.0, 0.5, 1.0) * float(abs((dissolveValue_1 - 1.0) - flrInput.dissolve) < 0.100000001490116119384765625)) * 1000.0);
        albedoValue = vec4(_425.x, _425.y, _425.z, albedoValue.w);
        vec3 _432 = min(albedoValue.xyz, vec3(8.0));
        albedoValue = vec4(_432.x, _432.y, _432.z, albedoValue.w);
    }
    float alphaValue = 1.0;
    float alphaCutoff = 1.0;
    if ((1.0 - flrInput.dissolve) < 1.0)
    {
        float dissolveValue_2 = SAMPLE_TEXTURE(dissolveTex, uv * _300.paramData6.x).x;
        alphaCutoff = 1.0 - flrInput.dissolve;
        alphaValue = alphaCutoff - dissolveValue_2;
    }
    if ((_300.paramIntData0.x & 1u) > 0u)
    {
        alphaCutoff *= (1.0 - _300.parameters0.w);
        alphaCutoff = 1.0 - clamp(alphaCutoff, 0.0, 1.0);
        alphaValue *= albedoValue.w;
        alphaValue -= alphaCutoff;
    }
    if (alphaValue < 0.0)
    {
        discard;
    }
    float _488;
    if ((_300.paramIntData0.x & 128u) > 0u)
    {
        _488 = 1.0 / max(albedoValue.w, 0.00999999977648258209228515625);
    }
    else
    {
        _488 = 1.0;
    }
    vec3 _500 = albedoValue.xyz * _488;
    albedoValue = vec4(_500.x, _500.y, _500.z, albedoValue.w);
    vec4 parameterValue = SAMPLE_TEXTURE1(parameterTex, uv, _330.frameParameters1.x);
    vec4 parameterOptional0Value = SAMPLE_TEXTURE1(parameterOptional0Tex, uv, _330.frameParameters1.x);
    float translucency = parameterOptional0Value.x;
    vec4 normalValue = SAMPLE_TEXTURE1(normalTex, uv, _300.normalParams.y + _330.frameParameters1.x);
    vec4 param = normalValue;
    vec3 normal = decodeNormalMap(param);
    vec2 _539 = normal.xy * _300.normalParams.x;
    normal = vec3(_539.x, _539.y, normal.z);
    flrOutput.normalsTS = normal;
    if (_300.normalParams.z > 0.0)
    {
        vec2 detailNormalUV = uv * _300.normalParams.w;
        vec4 param_1 = SAMPLE_TEXTURE1(detailNormalTex, detailNormalUV, _330.frameParameters1.x);
        vec3 detailNormal = decodeNormalMap(param_1);
        vec2 _567 = detailNormal.xy * _300.normalParams.z;
        detailNormal = vec3(_567.x, _567.y, detailNormal.z);
        detailNormal = normalize(detailNormal);
        vec3 param_2 = normal;
        vec3 param_3 = detailNormal;
        normal = blendNormalMaps(param_2, param_3);
    }
    normal = normalize(((flrInput.tangent.xyz * normal.x) + (flrInput.binormal * normal.y)) + (flrInput.normal * normal.z));
    flrOutput.albedo = vec4(albedoValue.xyz.x, albedoValue.xyz.y, albedoValue.xyz.z, flrOutput.albedo.w);
    flrOutput.albedo.w = albedoValue.w;
    vec3 param_4 = normal;
    vec3 param_5 = flrInput.normal;
    flrOutput.normals = vec4(encodeNormal(param_4), encodeNormal(param_5));
    flrOutput.materialId = _394.params0.w;
    vec3 colorCorrectionValue = SAMPLE_TEXTURE(colorCorrectionMaskTex, uv).xyz;
    float correctionMask = clamp(colorCorrectionValue.x * flrInput.color.w, 0.0, 1.0);
    uint param_6 = floatBitsToUint(_394.materialParams2.x);
    vec3 param_7 = decodeRGBA8uint(param_6).xyz;
    vec3 tintPerInstance = gammaToLinear(param_7);
    vec3 perInstanceColor = tintPerInstance * flrOutput.albedo.xyz;
    vec3 _658 = mix(flrOutput.albedo.xyz, perInstanceColor, vec3(correctionMask)) * _394.materialParams2.z;
    flrOutput.albedo = vec4(_658.x, _658.y, _658.z, flrOutput.albedo.w);
    vec3 correctedAlbedo = flrInput.color.zyx * flrOutput.albedo.xyz;
    vec3 param_8 = correctedAlbedo;
    float lum = getLuminance(param_8);
    correctedAlbedo = vec3(lum) + ((correctedAlbedo - vec3(lum)) * _300.tint.z);
    uint param_9 = floatBitsToUint(_300.tint.x);
    vec3 param_10 = decodeARGB8uint(param_9).xyz;
    correctedAlbedo = (correctedAlbedo * gammaToLinear(param_10)) * _300.tint.y;
    vec3 _703 = mix(flrOutput.albedo.xyz, correctedAlbedo, vec3(correctionMask));
    flrOutput.albedo = vec4(_703.x, _703.y, _703.z, flrOutput.albedo.w);
    vec3 parameterScale = _300.parameters0.xyz;
    flrOutput.parameter = vec4(parameterValue.xyz, 1.0);
    flrOutput.parameter.x = clamp((flrOutput.parameter.x * clamp(parameterScale.x, 0.0, 1.0)) + clamp(parameterScale.x - 1.0, 0.0, 1.0), 0.0, 1.0);
    flrOutput.parameter.y = clamp((flrOutput.parameter.y * clamp(parameterScale.y, 0.0, 1.0)) + clamp(parameterScale.y - 1.0, 0.0, 1.0), 0.0, 1.0);
    flrOutput.parameter.z = 1.0 - clamp((1.0 - flrOutput.parameter.z) * parameterScale.z, 0.0, 1.0);
    flrOutput.parameter.w = translucency;
    vec4 param_11 = flrInput.pos;
    vec4 param_12 = flrInput.prevPos;
    vec4 param_13 = _330.projectionOffsets;
    flrOutput.motionVector = getMotionVector(param_11, param_12, param_13);
    flrOutput.miscParams0 = _394.params0.x & 8191u;
    if ((_300.paramIntData0.x & 1024u) > 0u)
    {
        flrOutput.miscParams0 |= 16384u;
    }
    flrOutput.miscParams0 |= 32768u;
}

float encodeMaterialId(uint materialID)
{
    return unpackSnorm2x16(materialID).x;
}

float packUInt16(uint i)
{
    return 1.52587890625e-05 * float(i);
}

void main()
{
    bool _799;
    if (!gl_FrontFacing)
    {
        _799 = (_300.paramIntData0.x & 2u) == 0u;
    }
    else
    {
        _799 = gl_FrontFacing;
    }
    float normalSign = _799 ? 1.0 : (-1.0);
    FlrInput flrInput;
    flrInput.uv0 = inUV0;
    flrInput.normal = inNormal * normalSign;
    flrInput.tangent = inTangent;
    vec3 _820 = flrInput.tangent.xyz * normalSign;
    flrInput.tangent = vec4(_820.x, _820.y, _820.z, flrInput.tangent.w);
    flrInput.binormal = inBinormal * normalSign;
    flrInput.color = inColor;
    flrInput.posWS = inPosWS;
    flrInput.pos = inPosition;
    flrInput.prevPos = inPrevPosition;
    flrInput.dissolve = inDissolve.w;
    FlrInput param = flrInput;
    FlrOutput param_1;
    flrMain(param, param_1);
    FlrOutput flrOutput = param_1;
    outAlbedo = flrOutput.albedo;
    outNormals = flrOutput.normals;
    outParameter = flrOutput.parameter;
    outMotionVector = vec4(flrOutput.motionVector.x, flrOutput.motionVector.y, outMotionVector.z, outMotionVector.w);
    uint param_2 = flrOutput.materialId;
    outMotionVector.z = encodeMaterialId(param_2);
    uint param_3 = flrOutput.miscParams0;
    outMotionVector.w = packUInt16(param_3);
}

